__title__ = "ampharos"
__author__ = "Bijij"
__license__ = "MIT"
__copyright__ = "Copyright 2019-present Bijij"
__version__ = "2.0.1"

from . import tables as tables
from .search import *
from .setup import setup_ampharos as setup_ampharos
